'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));

const adapt = (componentSelector, overrideProps) => {
    return class Adapter extends React.Component {
        constructor() {
            super(...arguments);
            this.ref = null;
            this.eventListeners = {};
        }
        componentDidMount() {
            this.setUpEventListeners();
            this.updatePropertiesForRichData();
        }
        componentDidUpdate() {
            this.updatePropertiesForRichData();
        }
        componentWillUnmount() {
            this.removeEventListeners();
        }
        mapKeyToEventName(key) {
            if (this.shouldKeyBeMapped(key)) {
                return overrideProps[key];
            }
            return key.substr(2).toLowerCase();
        }
        mapKeyToPropertyName(key) {
            if (this.shouldKeyBeMapped(key)) {
                return overrideProps[key];
            }
            return key;
        }
        shouldKeyBeMapped(key) {
            return overrideProps ? !!overrideProps[key] : false;
        }
        removeEventListeners() {
            for (let key in this.eventListeners) {
                const handler = this.eventListeners[key];
                this.ref.removeEventListener(this.mapKeyToEventName(key), handler);
            }
            this.eventListeners = null;
        }
        setUpEventListeners() {
            for (let key in this.props) {
                const handler = this.props[key];
                if (key.indexOf(`on`) === -1 && !this.shouldKeyBeMapped(key)) {
                    continue;
                }
                if (typeof handler !== `function`) {
                    console.warn(`The prop ${key} is not a function.`);
                    continue;
                }
                this.eventListeners[key] = handler;
                this.ref.addEventListener(this.mapKeyToEventName(key), handler);
            }
        }
        updatePropertiesForRichData() {
            for (let key in this.props) {
                const data = this.props[key];
                if (!this.isRichData(data)) {
                    continue;
                }
                this.ref[this.mapKeyToPropertyName(key)] = data;
            }
        }
        filterOutRichData(props) {
            return Object.keys(props).reduce((filteredProps, propKey) => {
                const value = props[propKey];
                if (!this.isRichData(value)) {
                    filteredProps[propKey] = value;
                }
                return filteredProps;
            }, {});
        }
        isRichData(prop) {
            return typeof prop === "object";
        }
        render() {
            return React.createElement(componentSelector, Object.assign({ ref: (ref) => (this.ref = ref) }, this.filterOutRichData(this.props)));
        }
    };
};

exports.adapt = adapt;
